<?php

namespace App\Http\Controllers\Admin;

use App\Http\Controllers\Controller;
use App\Models\brand;
use App\Models\Category;
use App\Models\Product;
use App\Models\Product_Image;
use App\Models\ProductRating;
use App\Models\SubCategory;
use App\Models\TempImage;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Validator;
use Image;

class ProductController extends Controller
{
    /**
     * Display a listing of the resource.
     */
    public function index(Request $request)
    {
        $products = Product::latest('id')->with('Product_Image');

        if ($request->get('keyword') != "") {
            $products = $products->where('title', 'like', '%' . $request->keyword . '%');
        }

        $products = $products->paginate(10);
        return view('admin.product.index', compact('products'));
    }

    /**
     * Show the form for creating a new resource.
     */
    public function create()
    {
        $categories = Category::orderBy('name', 'ASC')->get();
        $brands = brand::orderBy('name', 'ASC')->get();

        return view('admin.product.create', compact('categories', 'brands'));
    }

    /**
     * Store a newly created resource in storage.
     */
    public function store(Request $request)
    {
        $rules = [
            'title' => ['required', 'max:255'],
            'product_size' => ['required', 'numeric'],
            'product_color' => ['required', 'max:255'],
            'slug' => ['required', 'unique:products', 'max:255'],
            'price' => ['required', 'numeric'],
            'sku' => ['required', 'unique:products'],
            'track_qty' => ['required', 'in:Yes,No'],
            'category' => ['required', 'numeric'],
            'is_featured' => ['required', 'in:Yes,No'],
        ];
        if (!empty($request->track_qty) && $request->track_qty == 'Yes') {
            $rules['qty'] = 'required|numeric';
        }
        $validator = Validator::make($request->all(), $rules);

        if ($validator->passes()) {
            $product = new Product();
            $product->title = $request->title;
            $product->slug = $request->slug;
            $product ->product_size = $request->product_size;
            $product ->product_color = $request->product_color;
            $product->short_description = $request->short_description;
            $product->description = $request->description;
            $product->shipping_returns = $request->shipping_returns;
            $product->price = $request->price;
            $product->compare_price = $request->compare_price;
            $product->sku = $request->sku;
            $product->barcode = $request->barcode;
            $product->track_qty = $request->track_qty;
            $product->qty = $request->qty;
            $product->status = $request->status;
            $product->category_id = $request->category;
            $product->sub_category_id = $request->sub_category;
            $product->brand_id = $request->brand;
            $product->is_featured = $request->is_featured;
            $product->related_products = (!empty($request->related_products)) ? implode(',',$request->related_products) : '';
            $product->save();

            //save Gallery Pics
            if (!empty($request->image_array)) {
                foreach ($request->image_array as $temp_image_id) {
                    $tempImageInfo = TempImage::find($temp_image_id);

                    $extArray = explode('.', $tempImageInfo->name);
                    $ext = last($extArray); // like jpg,gif,png etc

                    $productImage = new Product_Image();
                    $productImage->product_id = $product->id;
                    $productImage->image = 'null';
                    $productImage->save();

                    $imageName = $product->id . '-' . $productImage->id . '-' . time() . '.' . $ext;
                    $productImage->image = $imageName;
                    $productImage->save();

                    //Generate Product thumbnail

                    //Large Image
                    $sourcePath = public_path() . '/temp/' . $tempImageInfo->name;
                    $destPath = public_path() . '/uploads/product/large/' . $imageName;
                    $image = Image::make($sourcePath);
                    $image->resize(1400, null, function ($constraint) {
                        $constraint->aspectRatio();
                    });
                    $image->save($destPath);

                    //Small Image
                    $destPath = public_path() . '/uploads/product/small/' . $imageName;
                    $image = Image::make($sourcePath);
                    $image->fit(300, 300);

                    $image->save($destPath);
                }
            }

            $request->session()->flash('success', 'Product Created SuccessFully.');
            return response()->json([
                'status' => true,
                'message' => 'Product Created SuccessFully.'
            ]);
        } else {
            return response()->json([
                'status' => false,
                'errors' => $validator->errors()
            ]);
        }
    }

    /**
     * Display the specified resource.
     */
    public function show(Product $product)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     */
    public function edit($id, Request $request)
    {
        $product = Product::find($id);
        if (empty($product)) {
            return redirect()->route('product.index')->with('error', 'Product not Found');
        }

        //Fetch Product Images
        $productImage = Product_Image::where('product_id', $product->id)->get();

        $subCategories = SubCategory::where('category_id', $product->category_id)->get();
        $categories = Category::orderBy('name', 'ASC')->get();
        $brands = brand::orderBy('name', 'ASC')->get();

        //Fetch Related Product
        $relatedProducts = [];
        if($product->related_products != ''){
            $productArray = explode(',',$product->related_products);
            $relatedProducts = Product::wherein('id',$productArray)->get();
        }
        return view('admin.product.edit', compact('product', 'categories', 'brands', 'subCategories', 'productImage','relatedProducts'));
    }

    /**
     * Update the specified resource in storage.
     */
    public function update($id, Request $request)
    {
        $product = Product::find($id);

        $rules = [
            'title' => ['required', 'max:255'],
            'product_size' => ['required', 'numeric'],
            'product_color' => ['required', 'max:255'],
            'slug' => ['required', 'max:255', 'unique:products,slug,' . $product->id . ',id'],
            'price' => ['required', 'numeric'],
            'sku' => ['required', 'max:255', 'unique:products,sku,' . $product->id . ',id'],
            'track_qty' => ['required', 'in:Yes,No'],
            'category' => ['required', 'numeric'],
            'is_featured' => ['required', 'in:Yes,No'],
        ];
        if (!empty($request->track_qty) && $request->track_qty == 'Yes') {
            $rules['qty'] = 'required|numeric';
        }
        $validator = Validator::make($request->all(), $rules);

        if ($validator->passes()) {

            $product->title = $request->title;
            $product->slug = $request->slug;
            $product->short_description = $request->short_description;
            $product->description = $request->description;
            $product->shipping_returns = $request->shipping_returns;
            $product->price = $request->price;
            $product->compare_price = $request->compare_price;
            $product->sku = $request->sku;
            $product ->product_size = $request->product_size;
            $product ->product_color = $request->product_color;
            $product->barcode = $request->barcode;
            $product->track_qty = $request->track_qty;
            $product->qty = $request->qty;
            $product->status = $request->status;
            $product->category_id = $request->category;
            $product->sub_category_id = $request->sub_category;
            $product->brand_id = $request->brand;
            $product->is_featured = $request->is_featured;
            $product->related_products = (!empty($request->related_products)) ? implode(',',$request->related_products) : '';

            $product->save();

            //save Gallery Pics


            $request->session()->flash('success', 'Product Updated SuccessFully.');
            return response()->json([
                'status' => true,
                'message' => 'Product Updated SuccessFully.'
            ]);
        } else {
            return response()->json([
                'status' => false,
                'errors' => $validator->errors()
            ]);
        }
    }

    /**
     * Remove the specified resource from storage.
     */
    public function destroy($id, Request $request)
    {
        $product = Product::find($id);
        if (empty($product)) {
            $request->session()->flash('error', 'Product not Found');
            return response()->json([
                'status' => false,
                'notFound' => true,
            ]);
        }
        $productImages = Product_Image::where('product_id', $id)->get();

        if (!empty($productImages)) {
            foreach ($productImages as $productImage) {
                File::delete(public_path('/uploads/product/large/' . $productImage->image));
                File::delete(public_path('/uploads/product/small/' . $productImage->image));
            }
            Product_Image::where('product_id', $id)->delete();
        }
        $product->delete();
        $request->session()->flash('success', 'Product Deleted SuccessFully');
        return response()->json([
            'status' => true,
            'message' => 'Product Deleted SuccessFully.'
        ]);
    }

    public function getProducts(Request $request){
        $tempProduct = [];
        if($request->term != ""){
            $products = Product::where('title','like','%'.$request->term.'%')->get();

            if($products != null){
                foreach($products as $product){
                    $tempProduct[] = array('id' => $product->id, 'text' => $product->title);
                }
            }
        }

        return response()->json([
            'tags' => $tempProduct,
            'status' => true,
        ]);
    }


    //rating function
    public function productRating()
    {
        $product_ratings = ProductRating::latest()->get();
        return view('admin.product.ratings', compact('product_ratings'));
    }


    //change rating status function
    public function changeRatingStatus(Request $request)
    {
        $product_rating         = productRating::find($request->id);
        $product_rating->status = $request->status;
        $product_rating->save();

        session()->flash('success', 'Record updated successfully.');

        return response()->json([
            'status'            => true,
            'message'           => 'Record updated successfully.'
        ]);
    }
}